@@ -12,7 +12,7 @@
  **   This file is distributed under the terms of the General Public
  **   License. See the file COPYING for more information.
  **
- **   $Id: calculator.cpp 4600 2011-04-29 20:49:33Z axel $
+ **   $Id: calculator.cpp 4685 2011-09-21 16:52:31Z axel $
  **
  **   This class provides different calculator functionalities
  **
@@ -339,7 +339,9 @@ void Calculator::slot_WaypointDelete(Waypoint* newWp)
 {
   // @AP: check, if waypoint to be deleted is selected. In this case a
   // deselection must be done
-  if ( selectedWp && selectedWp->origP == newWp->origP )
+  if ( selectedWp &&
+       selectedWp->origP == newWp->origP &&
+       selectedWp->taskPointIndex == newWp->taskPointIndex )
     {
       wpTouched = false;
       wpTouchCounter = 0;
@@ -842,6 +844,9 @@ void Calculator::slot_changePosition(int direction)
           lastPosition = selectedWp->origP;
         }
       break;
+    case MapMatrix::Position:
+      // Nothing to do, new position was already set.
+      break;
     }
 
   // Check new position that it lays inside the earth coordinates +-90, +-180 degrees
@@ -892,6 +897,7 @@ void Calculator::slot_changePosition(int direction)
   calcDistance();
   calcBearing();
   calcGlidePath();
+
   // calculate always when moving manually (big delta !)
   _reachablelist->calculate(true);
   //qDebug("Elevation: %d m",_globalMapContents->findElevation(lastPosition) );
@@ -940,6 +946,14 @@ void Calculator::slot_changePositionWp()
 }
 
 
+/** Called if position was moved by using mouse. */
+void Calculator::slot_changePosition(QPoint& newPosition)
+{
+  lastPosition = newPosition;
+  slot_changePosition(MapMatrix::Position);
+}
+
+
 /** increment McCready value */
 void Calculator::slot_McUp()
 {
