@@ -8,12 +8,12 @@
  **
  **   Copyright (c):  1999, 2000 by Heiner Lamprecht, Florian Ehinger
  **                   2008 modified by Josua Dietze
- **                   2008-2010 modified by Axel Pauli
+ **                   2008-2011 modified by Axel Pauli
  **
  **   This file is distributed under the terms of the General Public
  **   License. See the file COPYING for more information.
  **
- **   $Id: map.cpp 4454 2010-11-30 20:56:17Z axel $
+ **   $Id: map.cpp 4687 2011-09-23 15:52:34Z axel $
  **
  ***********************************************************************/
 
@@ -74,12 +74,13 @@ Map::Map(QWidget* parent) : QWidget(parent)
   _isEnable = false;  // Disable map redrawing at startup
   _isResizeEvent = false;
   _isRedrawEvent = false;
+  _mouseMoveIsActive = false;
   mapRot = 0;
   curMapRot = 0;
   heading = 0;
   bearing = 0;
   mode = northUp;
-  m_scheduledFromLayer = topLayer;
+  m_scheduledFromLayer = baseLayer;
   ShowGlider = false;
   setMutex(false);
 
@@ -93,16 +94,7 @@ Map::Map(QWidget* parent) : QWidget(parent)
     }
 
   // Set pixmaps first to the size of the parent
-  m_pixBaseMap        = QPixmap( parent->size() );
-  m_pixAeroMap        = QPixmap( parent->size() );
-  m_pixNavigationMap  = QPixmap( parent->size() );
-  m_pixInformationMap = QPixmap( parent->size() );
-  m_pixPaintBuffer    = QPixmap( parent->size() );
-
-  m_pixBaseMap.fill(Qt::white);
-  m_pixAeroMap.fill(Qt::white);
-  m_pixNavigationMap.fill(Qt::white);
-  m_pixInformationMap.fill(Qt::white);
+  m_pixPaintBuffer = QPixmap( parent->size() );
   m_pixPaintBuffer.fill(Qt::white);
 
   QPalette palette;
@@ -130,7 +122,6 @@ Map::Map(QWidget* parent) : QWidget(parent)
   _glider = GeneralConfig::instance()->loadPixmap("gliders80pix-15.png");
 }
 
-
 Map::~Map()
 {
   qDeleteAll(airspaceRegionList);
@@ -139,7 +130,6 @@ Map::~Map()
 /**
  * Display Info about Airspace items
 */
-
 void Map::__displayAirspaceInfo(const QPoint& current)
 {
   if( mutex() || !isVisible() )
@@ -244,7 +234,6 @@ bool Map::__zoomButtonPress(const QPoint& point)
 /**
  * Display detailed Info about an airfield, a glider site or a waypoint.
 */
-
 void Map::__displayDetailedItemInfo(const QPoint& current)
 {
   if( mutex() )
@@ -256,17 +245,17 @@ void Map::__displayDetailedItemInfo(const QPoint& current)
   bool found = false;
 
   // Radius for Mouse Snapping
-  int delta=0, dX=0, dY=0;
+  int delta = 0, dX = 0, dY = 0;
 
   // define lists to be used for searching
   int searchList[] =
     {
       MapContents::AirfieldList,
-      MapContents::GliderSiteList,
+      MapContents::GliderfieldList,
       MapContents::OutLandingList
     };
 
-  wayPoint *w = static_cast<wayPoint *> (0);
+  Waypoint *w = static_cast<Waypoint *> (0);
 
   // scale uses unit meter/pixel
   double cs = _globalMapMatrix->getScale(MapMatrix::CurrentScale);
@@ -278,17 +267,17 @@ void Map::__displayDetailedItemInfo(const QPoint& current)
   delta = 25;
 #endif
 
-  // Manhattan-distance to found point.
-  int lastDist=2*delta+1;
+  // Manhattan distance is used to found the point.
+  int lastDist = 2 * delta + 1;
 
   // snapRect about the touched position
-  QRect snapRect( current.x()-delta, current.y()-delta, 2*delta, 2*delta );
+  QRect snapRect( current.x() - delta, current.y() - delta, 2 * delta, 2 * delta );
 
   //qDebug( "SnapRect %dx%d, delta=%d, w=%d, h=%d",
   //        current.x()-delta, current.y()-delta, delta, 2*delta, 2*delta );
 
-  // @AP: On map scale higher as 1024 we don't evolute anything
-  for (int l = 0; l < 3 && cs < 1024.0; l++)
+  // @AP: On map scale higher as 1024 we don't evaluate anything
+  for( int l = 0; l < 3 && cs < 1024.0; l++ )
     {
       for(unsigned int loop = 0;
           loop < _globalMapContents->getListLength(searchList[l]); loop++)
@@ -302,22 +291,23 @@ void Map::__displayDetailedItemInfo(const QPoint& current)
           QString siteIcao;
           QString siteDescription;
           short siteType;
-          double siteFrequency;
+          float siteFrequency;
           WGSPoint siteWgsPosition;
           QPoint sitePosition;
-          uint siteElevation;
+          float siteElevation;
           QPoint curPos;
           QString siteComment;
+          QString siteCountry;
 
           if( searchList[l] == MapContents::AirfieldList )
             {
               // Fetch data from airport list
-              site = _globalMapContents->getAirport(loop);
+              site = _globalMapContents->getAirfield(loop);
             }
-          else if( searchList[l] == MapContents::GliderSiteList )
+          else if( searchList[l] == MapContents::GliderfieldList )
             {
               // fetch data from glider site list
-              site = _globalMapContents->getGlidersite(loop);
+              site = _globalMapContents->getGliderfield(loop);
             }
           else if( searchList[l] == MapContents::OutLandingList )
             {
@@ -359,12 +349,13 @@ void Map::__displayDetailedItemInfo(const QPoint& current)
               siteIcao = site->getICAO();
               siteDescription = site->getName();
               siteType = site->getTypeID();
-              siteFrequency = site->getFrequency().toDouble();
+              siteFrequency = site->getFrequency();
               siteWgsPosition = site->getWGSPosition();
               sitePosition = site->getPosition();
               siteElevation = site->getElevation();
               const Runway &siteRunway = site->getRunway();
               siteComment = site->getComment();
+              siteCountry = site->getCountry();
 
               w = &wp;
               w->name = siteName;
@@ -380,6 +371,7 @@ void Map::__displayDetailedItemInfo(const QPoint& current)
               w->runway = siteRunway.direction;
               w->length = siteRunway.length;
               w->comment = siteComment;
+              w->country = siteCountry;
 
               found = true;
               lastDist = dX+dY;
@@ -393,26 +385,20 @@ void Map::__displayDetailedItemInfo(const QPoint& current)
     }
 
   // let's show waypoints
-  // @AP: On map scale higher as 1024 we don't evolute anything
+  // @AP: On map scale higher as 1024 we don't evaluate anything
+  QList<Waypoint>& wpList = _globalMapContents->getWaypointList();
 
-  QList<wayPoint>& wpList = _globalMapContents->getWaypointList();
-
-  for (int i = 0; i < wpList.count(); i++)
+  for( int i = 0; i < wpList.count() && cs < 1024.0; i++ )
     {
-      if (cs > 1024.0)
-        {
-          break;
-        }
-
-      wayPoint& wp = wpList[i];
+      Waypoint& wp = wpList[i];
 
       // consider only points, which are to drawn on the map
-      if( _globalMapMatrix->isWaypoint2Draw( wp.importance  ) )
+      if( _globalMapMatrix->isWaypoint2Draw( wp.priority  ) )
         {
           continue;
         }
 
-      QPoint sitePos (_globalMapMatrix->map(wp.projP));
+      QPoint sitePos( _globalMapMatrix->map( wp.projP ) );
 
       if( ! snapRect.contains(sitePos) )
         {
@@ -424,9 +410,9 @@ void Map::__displayDetailedItemInfo(const QPoint& current)
       dY = abs(sitePos.y() - current.y());
 
       // Abstand entspricht der Icon-Groesse
-      if (dX < delta && dY < delta)
+      if( dX < delta && dY < delta )
         {
-          if (found && ((dX+dY) > lastDist))
+          if( found && ((dX + dY) > lastDist) )
             { // subtle difference with airfields: replace already
               // found waypoint if we find a waypoint at the same
               // distance.
@@ -440,31 +426,75 @@ void Map::__displayDetailedItemInfo(const QPoint& current)
 
           lastDist = dX+dY;
 
-          if (lastDist< (delta/3)) //if we're very near, stop searching the list
+          if( lastDist < (delta / 3) )
             {
+              // if we're very near, stop searching the list
               break;
             }
         }
     }
 
-  if (found)
+  // Search in the current task, if waypoints can be found. This waypoints have
+  // the most important display priority.
+  FlightTask* task = (FlightTask*) _globalMapContents->getCurrentTask();
+
+  if( task != static_cast<FlightTask *> (0) && cs < 1024.0 )
+    {
+      QList<TaskPoint*>& tpList = task->getTpList();
+
+      for( int i = 0; i < tpList.size(); i++ )
+        {
+          TaskPoint* tp = tpList[i];
+
+          QPoint sitePos( _globalMapMatrix->map( tp->projP ) );
+
+          if( ! snapRect.contains(sitePos) )
+            {
+              // @AP: Point lays outside of snap rectangle, we ignore it
+              continue;
+            }
+
+          dX = abs(sitePos.x() - current.x());
+          dY = abs(sitePos.y() - current.y());
+
+          if( dX < delta && dY < delta )
+            {
+              if( found && ((dX + dY) > lastDist) )
+                { // subtle difference with airfields: replace already
+                  // found waypoint if we find a waypoint at the same
+                  // distance.
+                  continue; // the point we found earlier was closer
+                }
+
+              found = true;
+              w = tp;
+              lastDist = dX+dY;
+
+              if( lastDist < (delta / 3) )
+                {
+                  break;
+                }
+            }
+        }
+    }
+
+  if( found )
     {
       // qDebug ("Waypoint: %s", w->name.toLatin1().data() );
-      emit waypointSelected(w);
+      emit waypointSelected( w );
       return;
     }
 
   // @ee maybe we can show airspace info
-  if (!found)
+  if( !found )
     {
-      __displayAirspaceInfo(current);
+      __displayAirspaceInfo( current );
     }
 }
 
-
 void Map::mousePressEvent(QMouseEvent* event)
 {
-  // qDebug("Map::mousePressEvent():");
+  // qDebug() << "Map::mousePressEvent(): Pos=" << event->pos();
 
   if( mutex() )
     {
@@ -472,49 +502,108 @@ void Map::mousePressEvent(QMouseEvent* event)
       return;
     }
 
-  // qDebug("Map::mousePressEvent");
   switch (event->button())
     {
-    case Qt::RightButton: // press and hold generates mouse RightButton
-      // qDebug("RightButton");
-      break;
-    case Qt::LeftButton: // press generates mouse LeftButton immediately
-      // qDebug("LeftButton");
-      if( __zoomButtonPress( event->pos() ) )
-          {
-            break;
-          }
+      case Qt::RightButton: // press and hold generates mouse RightButton
+        // qDebug("MP-RightButton");
+        break;
 
-      __displayDetailedItemInfo( event->pos() );
-      break;
-    case Qt::MidButton:
-      // qDebug("MidButton");
-      break;
+      case Qt::LeftButton: // press generates mouse LeftButton immediately
+        // qDebug("MP-LeftButton");
+        _beginMapMove = event->pos();
+        event->accept();
+        break;
 
-    default:
-      break;
+      case Qt::MidButton:
+        // qDebug("MP-MidButton");
+        break;
+
+      default:
+        break;
     }
 }
 
-void
-Map::mouseReleaseEvent( QMouseEvent* event )
+void Map::mouseMoveEvent( QMouseEvent* event )
 {
-  // qDebug("Map::mouseReleaseEvent():");
+  if( _mouseMoveIsActive == false &&
+     ( GpsNmea::gps->getGpsStatus() != GpsNmea::validFix || calculator->isManualInFlight() ) )
+    {
+      QPoint dist = _beginMapMove - event->pos();
+
+      if( dist.manhattanLength() > 10 )
+        {
+          // On the N810/N900 we get a lot of move events also on a single
+          // mouse press and release action. If we do not filter that,
+          // all other mouse actions bound on release mouse are blocked.
+          _mouseMoveIsActive = true;
+          QApplication::setOverrideCursor(QCursor(Qt::SizeAllCursor));
+          event->accept();
+        }
+    }
+}
+
+void Map::mouseReleaseEvent( QMouseEvent* event )
+{
+  // qDebug() << "Map::mouseReleaseEvent(): Pos=" << event->pos();
+
+  if( _mouseMoveIsActive )
+    {
+      // User has released the mouse button after a mouse move. We move
+      // the map to the new position if we are in manual mode or the GPS
+      // is not connected
+      _mouseMoveIsActive = false;
+
+      QApplication::restoreOverrideCursor();
+      QPoint dist = _beginMapMove - event->pos();
+      QPoint center( width()/2, height()/2 );
+
+      center += dist;
+
+      // Center Map to new center point
+      _globalMapMatrix->centerToPoint( center );
+      QPoint newPos = _globalMapMatrix->mapToWgs( center );
+
+      // Coordinates are toggled, don't know why
+      curMANPos = QPoint(newPos.y(), newPos.x());
+      emit newPosition( curMANPos );
+      scheduleRedraw();
+      event->accept();
+      return;
+    }
+
   if( mutex() )
     {
-      // qDebug("Map::mouseReleaseEvent(): mutex is locked, returning");
+      //qDebug("Map::mouseReleaseEvent(): mutex is locked, returning");
       return;
     }
 
   switch( event->button() )
     {
-    case Qt::LeftButton:
-      // __displayAirspaceInfo(event->pos());
-      break;
+      case Qt::RightButton: // press and hold generates mouse RightButton
+        //qDebug("MR-RightButton");
+        break;
 
-    default:
-      break;
+      case Qt::LeftButton: // press generates mouse LeftButton immediately
+        //qDebug("MR-LeftButton");
+
+        if( __zoomButtonPress( event->pos() ) )
+          {
+            break;
+          }
+
+        __displayDetailedItemInfo( event->pos() );
+        break;
+
+      case Qt::MidButton:
+
+        // qDebug("MR-MidButton");
+        break;
+
+      default:
+        break;
     }
+
+  event->accept();
 }
 
 void Map::paintEvent(QPaintEvent* event)
@@ -532,8 +621,7 @@ void Map::paintEvent(QPaintEvent* event)
       qDebug("Map::paintEvent(): mutex is locked");
     }
 
-  // We copy always the content from the m_pixPaintBuffer to the paint
-  // device.
+  // We copy always the content from the m_pixPaintBuffer to the paint device.
   QPainter p(this);
   p.drawPixmap( event->rect().left(), event->rect().top(), m_pixPaintBuffer,
                 0, 0, event->rect().width(), event->rect().height() );
@@ -794,7 +882,7 @@ void Map::__drawGrid()
   gridP.end();
 }
 
-void Map::__drawPlannedTask( QPainter *taskP, QList<wayPoint*> &drawnWp )
+void Map::__drawPlannedTask( QPainter *taskP, QList<Waypoint*> &drawnWp )
 {
   FlightTask* task = (FlightTask*) _globalMapContents->getCurrentTask();
 
@@ -871,6 +959,8 @@ void Map::__redrawMap(mapLayer fromLayer, bool queueRequest)
 {
   static bool first = true; // mark first calling of method
 
+  static QSize lastSize; // Save the last used window size
+
   // qDebug("Map::__redrawMap from layer=%d", fromLayer);
 
   // First call after creation of object can pass
@@ -938,12 +1028,17 @@ void Map::__redrawMap(mapLayer fromLayer, bool queueRequest)
 
       zoomFactor = _globalMapMatrix->getScale(); //read back from matrix!
 
-      qDebug("MapMatrixSize: w=%d, h=%d", size().width(), size().height());
+      // qDebug("MapMatrixSize: w=%d, h=%d", size().width(), size().height());
 
       _globalMapMatrix->createMatrix(this->size());
 
-      // initialize the base pixmap
-      m_pixBaseMap = QPixmap( size() );
+      if( lastSize.isValid() == false || lastSize != size() )
+        {
+          // save the last used size
+          lastSize = size();
+          // reinitialize the base pixmap
+          m_pixBaseMap = QPixmap( size() );
+        }
 
       //actually start doing our drawing
       __drawBaseLayer();
@@ -1118,14 +1213,14 @@ void Map::__drawNavigationLayer()
   // Collect all drawn airfield and waypoint objects as reference
   // for later label drawing:
   QList<Airfield*> drawnAf;
-  QList<wayPoint*> drawnWp;
+  QList<Waypoint*> drawnWp;
 
   QPainter navP;
 
   navP.begin(&m_pixNavigationMap);
 
   _globalMapContents->drawList(&navP, MapContents::OutLandingList, drawnAf);
-  _globalMapContents->drawList(&navP, MapContents::GliderSiteList, drawnAf);
+  _globalMapContents->drawList(&navP, MapContents::GliderfieldList, drawnAf);
   _globalMapContents->drawList(&navP, MapContents::AirfieldList, drawnAf);
   __drawWaypoints(&navP, drawnWp);
   __drawPlannedTask(&navP, drawnWp);
@@ -1286,7 +1381,7 @@ void Map::slotRedrawMap()
 }
 
 /** Draws the waypoints of the waypoint catalog on the map */
-void Map::__drawWaypoints(QPainter* painter, QList<wayPoint*> &drawnWp)
+void Map::__drawWaypoints(QPainter* painter, QList<Waypoint*> &drawnWp)
 {
   extern MapConfig* _globalMapConfig;
 
@@ -1299,7 +1394,7 @@ void Map::__drawWaypoints(QPainter* painter, QList<wayPoint*> &drawnWp)
   Altitude alt;
   Distance dist;
 
-  QList<wayPoint>& wpList = _globalMapContents->getWaypointList();
+  QList<Waypoint>& wpList = _globalMapContents->getWaypointList();
 
   // load all configuration items once
   const bool showWpLabels   = GeneralConfig::instance()->getMapShowWaypointLabels();
@@ -1309,17 +1404,16 @@ void Map::__drawWaypoints(QPainter* painter, QList<wayPoint*> &drawnWp)
   // now step trough the waypoint list
   for( int i=0; i < wpList.count(); i++ )
     {
-      wayPoint& wp = wpList[i];
+      Waypoint& wp = wpList[i];
 
       // qDebug("wp=%s", wp.name.toLatin1().data());
 
       // isSelected is used for the currently selected target point
       bool isSelected = false;
 
-      if (calculator && calculator->getselectedWp() )
+      if( calculator && calculator->getselectedWp() )
         {
-          if (calculator->getselectedWp()->name == wp.name &&
-              calculator->getselectedWp()->origP == wp.origP)
+          if( *calculator->getselectedWp() == wp )
             {
               isSelected = true;
             }
@@ -1332,7 +1426,7 @@ void Map::__drawWaypoints(QPainter* painter, QList<wayPoint*> &drawnWp)
         }
 
       // Check if the waypoint is important enough for the current map scale.
-      if( _globalMapMatrix->isWaypoint2Draw( wp.importance ) && isSelected == false )
+      if( _globalMapMatrix->isWaypoint2Draw( wp.priority ) && isSelected == false )
         {
           // qDebug("Not important wp=%s", wp.name.toLatin1().data());
           continue;
@@ -1825,7 +1919,7 @@ void Map::__drawScale(QPainter& scaleP)
  */
 void Map::slotPosition(const QPoint& newPos, const int source)
 {
-  // qDebug("Map::slot_position x=%d y=%d", newPos.x(), newPos.y() );
+  //qDebug("Map::slot_position x=%d y=%d", newPos.x(), newPos.y() );
 
   if( !_isEnable )
     {
